%% This function reads a .csv file. 'n' is the minimum number of piles repetitions to be considered. It removes piles with less than 'n' repetitions
function [error,I,J,ci,yilij,weights,raw_data] = loadData_csv(filename,n)
error = '';
if ~exist(filename,'file')
    error = ['File ' filename ' does not exist.'];I=-1;J=-1;ci=-1;yilij=-1;weights=-1;raw_data=-1;
    return;
end

f = fopen(filename,'r');
line = fgetl(f);
raw = split(line);
while ischar(line)
    line = fgetl(f);
    if ischar(line) && strcmp(line,'')==0
        splited = split(line);
        if length(splited) == size(raw,2)
            raw = [raw;splited];
        else
            error = 'The selected data format is not valid.';I=-1;J=-1;ci=-1;yilij=-1;weights=-1;raw_data=-1;
            return;
        end
    end
end
for i = 1:size(raw,1)
    for j = 1:size(raw,2)
        if ~isnan(str2double(raw{i,j}))
            raw{i,j} = str2double(raw{i,j});
        end
    end
end
fclose(f);


if (size(raw,1) < 2 || size(raw,2) < 3)
    error = 'The selected data format is not valid.';I=-1;J=-1;ci=-1;yilij=-1;weights=-1;raw_data=-1;
    return;
end
IDs_raw = cell2mat(raw(2:end,1));
categories_raw = raw(2:end,2);

found = false;
for firstColumnWithData = 3:size(raw,2) % looking for first 0 or 1
    tmp = raw{2,firstColumnWithData};
    if (isfloat(tmp) && (tmp == 0 || tmp == 1))
        found = true;
        break;
    end
end
if ~found % didnt find any data on second row
    error = 'The selected data format is not valid.';I=-1;J=-1;ci=-1;yilij=-1;weights=-1;raw_data=-1;
    return;
end
found = false;
for lastColumnWithData = firstColumnWithData:size(raw,2)
    tmp = raw{2,lastColumnWithData};
    if (~isfloat(tmp) || (tmp ~= 0 && tmp ~= 1))
        found = true;
        break;
    end
end
if found
    lastColumnWithData = lastColumnWithData - 1;
end
for i = 2:length(IDs_raw)+1
    for j = firstColumnWithData:lastColumnWithData
        tmp = raw{i,j};
        if (~isfloat(tmp) || (tmp ~= 0 && tmp ~= 1))
            error = 'The selected data format is not valid.';I=-1;J=-1;ci=-1;yilij=-1;weights=-1;raw_data=-1;
            return;
        end
    end
end
yilij_raw = cell2mat(raw(2:end,firstColumnWithData:lastColumnWithData));
if length(IDs_raw)~=length(categories_raw) || length(IDs_raw)~=size(yilij_raw,1)
    error = 'The selected data format is not valid.';I=-1;J=-1;ci=-1;yilij=-1;weights=-1;raw_data=-1;
    return;
end
products_raw = raw(1,firstColumnWithData:lastColumnWithData);
raw_data.products = products_raw;
if n == 1
    raw_data.IDs(1) = IDs_raw(1);
    raw_data.categories{1}(1) = categories_raw(1);
    raw_data.yilij{1}(1,:) = yilij_raw(1,:);
    i=1;
    for ii=2:length(IDs_raw)
        if IDs_raw(ii) ~= raw_data.IDs(i)
            i = i+1;
            raw_data.IDs(i) = IDs_raw(ii);
            raw_data.categories{i} = categories_raw(ii);
            raw_data.yilij{i} = yilij_raw(ii,:);
        else
            raw_data.categories{i} = [raw_data.categories{i};categories_raw(ii)];
            raw_data.yilij{i} = [raw_data.yilij{i};yilij_raw(ii,:)];
        end
    end
    raw_data.raw = raw;
end

% Processing data

I=1;
weights=cell(1);
J=size(yilij_raw,2);

yilij=cell(1);

yilij{1}=[yilij{1};yilij_raw(1,:)];
weights{1}=[weights{1},1];
for li_raw=2:size(yilij_raw,1)
    ci=size(yilij{1},1);
    found=false;
    for li=1:ci
        if yilij_raw(li_raw,:)==yilij{1}(li,:)
            weights{1}(li)=weights{1}(li)+1;
            found=true;
            break;
        end
    end
    if found==false
        yilij{1}=[yilij{1};yilij_raw(li_raw,:)];
        weights{1}=[weights{1},1];
    end
end
ci=size(yilij{1},1);

if n>1
    new_weights=cell(1);
    new_yilij=cell(1);
    for li=1:ci
        if weights{1}(li)>=n
            new_weights{1}=[new_weights{1},weights{1}(li)];
            new_yilij{1}=[new_yilij{1};yilij{1}(li,:)];
        end
    end
    yilij=new_yilij;
    weights=new_weights;
    ci=size(yilij{1},1);
    
    % reducing raw data
    index = [];
    for li = 1:size(yilij_raw,1)
        for lii = 1:size(yilij{1},1)
            if isequal(yilij_raw(li,:),yilij{1}(lii,:))
                index = [index,li];
                break;
            end
        end 
    end
    raw = raw([1,index+1],:);
    if (size(raw,1) < 2)
        error = 'The data becomes empty with the chosen minimum number of equal piles. Chose a lower number.';I=-1;J=-1;ci=-1;yilij=-1;weights=-1;raw_data=-1;
        return;
    end
    IDs_raw = cell2mat(raw(2:end,1));
    categories_raw = raw(2:end,2);
    yilij_raw = cell2mat(raw(2:end,firstColumnWithData:lastColumnWithData));
    
    raw_data.IDs(1) = IDs_raw(1);
    raw_data.categories{1}(1) = categories_raw(1);
    raw_data.yilij{1}(1,:) = yilij_raw(1,:);
    i=1;
    for ii=2:length(IDs_raw)
        if IDs_raw(ii) ~= raw_data.IDs(i)
            i = i+1;
            raw_data.IDs(i) = IDs_raw(ii);
            raw_data.categories{i} = categories_raw(ii);
            raw_data.yilij{i} = yilij_raw(ii,:);
        else
            raw_data.categories{i} = [raw_data.categories{i};categories_raw(ii)];
            raw_data.yilij{i} = [raw_data.yilij{i};yilij_raw(ii,:)];
        end
    end
    raw_data.raw = raw;
    
end

for li=1:ci
    for li2=li+1:ci
        if yilij{1}(li,:)==yilij{1}(li2,:)
            disp('ERROR: There are equal piles into yilij!')
        end
    end
end

function [strings] = split(line)
strings{1} = '';
i = 1;
words = 1;
while i <= length(line)
    c = line(i);
    if c == ','
        words = words+1;
        strings{words} = '';
    elseif c == '"'
        reached_end = false;
        n = 0;
        while c == '"'
            n = n+1;
            i = i+1;
            if i > length(line)
                reached_end = true;
                break;
            end
            c = line(i);
        end
        if mod(n,2)==0 % put quotes and continue
            i = i-1;
            strings{words} = [strings{words} char(ones(1,n/2)*'"')];
        else % read until an odd number of quotes is found
            strings{words} = [strings{words} char(ones(1,(n-1)/2)*'"')];
            if reached_end
                break;
            end
            found = false;
            while(~found)
                while c ~= '"'
                    strings{words} = [strings{words} c];
                    i = i+1;
                    if i > length(line)
                        reached_end = true;
                        disp('Unbalanced quotes')
                        break;
                    end
                    c = line(i);
                end
                if reached_end
                    break;
                end
                n = 0;
                while c == '"'
                    n = n+1;
                    i = i+1;
                    if i > length(line)
                        reached_end = true;
                        break;
                    end
                    c = line(i);
                end
                if mod(n,2) == 0
                    strings{words} = [strings{words} char(ones(1,n/2)*'"')];
                else
                    found = true;
                    strings{words} = [strings{words} char(ones(1,(n-1)/2)*'"')];
                    i = i-1;
                end
            end
        end
    else
        strings{words} = [strings{words} c];
    end
    i=i+1;
end