%% Local search procedure of VNS
function [localBestZ,localBestPmj,localBestXimli]=local_search(ximli,pmj,yilij,weights)
%% Inputs:
% ximli: cell where each value equals to 1 if pile li is assigned to summary pile m
% pmj: summary piles
% yilij: piles
% weights: number of times each pile was created

%% Output:
% localBestZ: total miscoverings of the returned solution
% localBestPmj: best summary piles found
% localBestXimli: best assignment for the returned solution

%%
[K,J]=size(pmj{1});
I=length(yilij);
ci=zeros(1,I);
for i=1:I
    ci(i)=size(yilij{i},1);
end

localBestPmj=pmj;
localBestZ=1e15;

no_change=false;
while no_change==false
    %% Computing best summary piles given ximli
    count_mj=zeros(K,J); % number of times consumers put product j in piles
                         % that are covered by the summary pile m
    
    covering_count_m=zeros(K,1); % number of piles that summary pile m is
                                 % covering
    for m=1:K
        for i=1:I
            for li=1:ci(i)
                if ximli{i}(m,li)==1
                    covering_count_m(m)=covering_count_m(m)+weights{1}(li);
                    for j=1:J
                        if yilij{i}(li,j)==1
                            count_mj(m,j)=count_mj(m,j)+weights{1}(li);
                        end
                    end
                end
            end
        end
    end
    
    for m=1:K
        for j=1:J
            if covering_count_m(m)~=0
                count_mean=count_mj(m,j)/covering_count_m(m);
                if count_mean >= 0.5
                    pmj{1}(m,j)=1;
                else
                    pmj{1}(m,j)=0;
                end
            end
        end
    end
    
    %% Rearranging piles according to cost (minimizing miscoverings)
    
    new_ximli=cell(I,1);
    
    mismatches=cell(I,1);
    
    Z=0;
    
    for i=1:I
        mismatches{i} = J-(pmj{1}*yilij{i}' + (1-pmj{1})*(1-yilij{i}'));
        
        new_ximli(i,1)={zeros(K,ci(i))};
        
        for li=1:ci(i)
            [val,id]=min(mismatches{i}(:,li));
            Z=Z+val*weights{1}(li);
            new_ximli{i}(id,li)=1;
        end
    end
    
    %% Treating results
    if new_ximli{1}==ximli{1}
        no_change=true;
    else
        ximli=new_ximli;
    end
    
    if Z < localBestZ
        localBestZ=Z;
        localBestPmj=pmj;
    end
end

%% If there's an empty summary pile then let it be equal to the pile of maximum ocurrence that it covers. Then, rearrange piles one more time
for m=1:K
    if sum(localBestPmj{1}(m,:))==0 % if empty pile
        max_ocurrence=0;
        for i=1:I
            i_max_ocurrence=i;
            li_max_ocurrence=randi(ci(i));
            for li=1:ci(i)
                if ximli{i}(m,li)==1 && weights{1}(li)>max_ocurrence
                    max_ocurrence=weights{1}(li);
                    i_max_ocurrence=i;
                    li_max_ocurrence=li;
                end
            end
            localBestPmj{1}(m,:)=yilij{i_max_ocurrence}(li_max_ocurrence,:);
        end
    end
end
[localBestXimli,localBestZ]=get_cost(localBestPmj,yilij,weights);

end