%% Local search procedure of VNS when piles from the same individual must be covered by different summay piles
function [localBestZ,localBestPmj,localBestXimli]=local_searchOPPSP(ximli,pmj,yilij)
%% Inputs:
% ximli: cell where each value equals to 1 if pile li is assigned to summary pile m
% pmj: summary piles
% yilij: piles

%% Output:
% localBestZ: total miscoverings of the returned solution
% localBestPmj: best summary piles found
% localBestXimli: best assignment for the returned solution

%%
[K,J]=size(pmj{1});
I=length(yilij);
ci=zeros(1,I);
for i=1:I
    ci(i)=size(yilij{i},1);
end

localBestPmj=pmj;
localBestZ=1e15;

no_change=false;
while no_change==false
    %% Computing best summary piles given ximli
    count_mj=zeros(K,J); % number of times consumers put product j in piles
                         % that are covered by the summary pile m
    
    covering_count_m=zeros(K,1); % number of piles that summary pile m is
                                 % covering
    for m=1:K
        for i=1:I
            for li=1:ci(i)
                if ximli{i}(m,li)==1
                    covering_count_m(m)=covering_count_m(m)+1;
                    for j=1:J
                        if yilij{i}(li,j)==1
                            count_mj(m,j)=count_mj(m,j)+1;
                        end
                    end
                end
            end
        end
    end
    
    for m=1:K
        for j=1:J
            if covering_count_m(m)~=0
                count_mean=count_mj(m,j)/covering_count_m(m);
                if count_mean >= 0.5
                    pmj{1}(m,j)=1;
                else
                    pmj{1}(m,j)=0;
                end
            end
        end
    end
    
    %% Rearranging piles according to cost (minimizing miscoverings)
    
    new_ximli=cell(I,1);
    
    Z=0;
    
    for i=1:I
        mismatches = J-(pmj{1}*yilij{i}' + (1-pmj{1})*(1-yilij{i}'));
        
        new_ximli(i,1)={zeros(K,ci(i))};
        
        visited_yilij = zeros(ci(i),1);
        visited_pmj = zeros(K,1);
        
        mismatches_row = zeros(ci(i)*K,3); % every row means: [cost to assign li to m, li, m]
        for li=1:ci(i)
            for m = 1:K
                mismatches_row(m+K*(li-1),:) = [mismatches(m,li),li,m];
            end
        end
        [~,perm] = sort(mismatches_row(:,1)); % sorts according to the cost of linking each pile to each summary pile
        mismatches_row = mismatches_row(perm,:);
        ii=1;
        while sum(visited_yilij)<ci(i)
            if visited_yilij(mismatches_row(ii,2)) == 0 && visited_pmj(mismatches_row(ii,3)) == 0
                visited_pmj(mismatches_row(ii,3)) = 1;
                visited_yilij(mismatches_row(ii,2)) = 1;
                Z = Z + mismatches_row(ii,1);
                new_ximli{i}(mismatches_row(ii,3),mismatches_row(ii,2))=1;
            end
            ii = ii + 1;
        end
    end
    
    %% Treating results
    
    no_change = true;
    for i = 1:I
        if new_ximli{i}~=ximli{i}
            no_change=false;
            ximli=new_ximli;
        end
    end
    
    if Z < localBestZ
        localBestZ=Z;
        localBestPmj=pmj;
    end
end

%% If there's an empty summary pile then add the object of maximum ocurrence in piles covered by it. Then, rearrange piles one more time
for m=1:K
    if sum(localBestPmj{1}(m,:))==0 % if empty pile
        s = zeros(1,J);
        for i = 1:I
            for li = 1:ci(i)
                if ximli{i}(m,li) == 1
                    for j = 1:J
                        s(j) = s(j) + yilij{i}(li,j);
                    end
                end
            end
        end
        [~,j] = max(s);
        localBestPmj{1}(m,j) = 1;
    end
end
[localBestXimli,localBestZ]=get_costOPPSP(localBestPmj,yilij);

end