%% This function saves results in file "filename"
function [error] = save_default(filename,raw_data,processed_data,results)
error = '';
try
    if exist(filename,'file')
        disp(['deleting ' filename]);
        delete(filename);
        disp('done.');
    end
catch
    error = ['The file ' filename ' is not writable.'];
    return;
end
%% SHEET 1
bold_cells1 = [];
sub_cells1 = [];
sheet1_name = 'Summary';
sheet1{2,1} = 'Category Covering 1.0'; bold_cells1 = [bold_cells1;2,1];
sheet1{4,1} = 'Software by:';
sheet1{5,1} = '(withheld through the review process)';
sheet1{7,1} = 'Key data:'; bold_cells1 = [bold_cells1;7,1];
I = length(raw_data.yilij);
sheet1{8,1} = I; sheet1{8,2} = 'individuals';
sum_ci=0;
for i = 1:I
    sum_ci = sum_ci + size(raw_data.yilij{i},1);
end
sheet1{9,1} = sum_ci; sheet1{9,2} = 'piles made in the data';
sheet1{10,1} = size(processed_data.yilij{1},1); sheet1{10,2} = 'unique piles';
J = size(raw_data.yilij{1},2);
sheet1{11,1} = J; sheet1{11,2} = 'objects sorted';
sheet1{12,1} = results.seed(1,1); sheet1{12,2} = 'Seed';
sheet1{14,1} = 'Summary:'; bold_cells1 = [bold_cells1;14,1];
sheet1{15,1} = 'Selection of Number of Piles'; sub_cells1 = [sub_cells1;15,1];
sheet1{16,1} = 'This sheet is used to help you select the number of summary piles appropriate for your data.';
sheet1{18,1} = 'Detailed piles summary'; sub_cells1 = [sub_cells1;18,1];
sheet1{19,1} = 'For each value of K, the best solution is presented.';
status = xlswrite(filename,sheet1,1);
if status == 0
    error = ['The file ' filename ' is not writable.  It may be locked by another process.'];
    return;
end

%% SHEET 2
bold_cells2 = [];
percent_cells2 = [];
sheet2_name = 'Selection of # of Summary piles';
sheet2{1,1} = 'Selection of the Number of Summary piles'; bold_cells2 = [bold_cells2;1,1];
sheet2{4,1} = 'Number of Summary Piles';
sheet2{3,2} = 'Execution';
runs  = results.runs;
for r = 1:runs
    sheet2{4,1+r} = r;
end
sheet2{4,2+runs} = 'Min';
sheet2{3,2+runs} = 'Best Solution';
sheet2{4,3+runs} = 'Improvement';
sheet2{4,4+runs} = 'Category Covering Percentage';
sheet2{4,5+runs} = 'Improvement in percentage';
maxK = results.maxK;
minK = results.minK;
Ks = maxK-minK+1;
bestSample = zeros(1,Ks);
for k=1:Ks;
    sheet2{4+k,1} = minK+k-1;
    for r = 1:runs
        sheet2{4+k,1+r} = results.Z(k,r);
    end
    [sheet2{4+k,2+runs},bestSample(k)] = min(results.Z(k,:));
    if k==1
        sheet2{4+k,3+runs} = '-';
        sheet2{4+k,5+runs} = '-';
    else
        sheet2{4+k,3+runs} = sheet2{3+k,2+runs} - sheet2{4+k,2+runs};
        if sheet2{4+k,3+runs} == 0
            sheet2{4+k,5+runs} = 0; percent_cells2 = [percent_cells2;4+k,5+runs];
        else
            sheet2{4+k,5+runs} = sheet2{4+k,3+runs}/sheet2{3+k,2+runs}; percent_cells2 = [percent_cells2;4+k,5+runs];
        end
    end
end
status = xlswrite(filename,sheet2,2);
if status == 0
    error = ['The file ' filename ' is not writable.  It may be locked by another process.'];
    return;
end

%% SHEET 3
bold_cells3 = [];
percent_cells3 = [];
sheet3_name = 'Detailed piles summary';
sheet3{1,1} = 'Summary Piles Output'; bold_cells3 = [bold_cells3;1,1];
sheet3{3,1} = '1: Item is part of summary pile';
sheet3{4,1} = '0: Item is not part of summary pile';
sheet3{5,1} = 'Covering rate: % of piles predicted by the summary pile';

line = 7;
for k = 1:Ks
    line = line + 1;
    sheet3{line,1} = [num2str(k-1+minK) ' piles solution']; bold_cells3 = [bold_cells3;line,1];
    sheet3{line,2} = 'Covering Rate';
    for j = 1:J
        sheet3{line,2+j} = raw_data.products{j};
    end
    line = line + 1;
    covRate = results.covRate{k,bestSample(k)};
    pmj = results.pmj{k,bestSample(k)};
    for m = 1:minK+k-1
        sheet3{line,2} = covRate(m); percent_cells3 = [percent_cells3;line,2];
        for j = 1:J
            sheet3{line,2+j} = pmj(m,j);
        end
        line = line + 1;
    end
end
status = xlswrite(filename,sheet3,3);
if status == 0
    error = ['The file ' filename ' is not writable.  It may be locked by another process.'];
    return;
end

%% BOLD, UNDERLINE and PERCENTAGE
try
    Excel = actxserver('Excel.Application');
catch
    error = 'Cannot edit cells format. Check if Excel Application is installed in your system.';
    Excel = [];
end
if ~isempty(Excel)
    ExcelVersion = str2num(Excel.Version);
    if ExcelVersion < 12
        error = 'Cannot edit cells format. Your Excel Application version may not support *.xlsx files.';
    else
        Workbook = invoke(Excel.Workbooks,'Open', filename);
%         set(Excel, 'Visible', 1);
        % Sheet 1
        sheet = get(Excel.Worksheets,'Item', 1);
        invoke(sheet, 'Activate');
        set(sheet, 'Name', sheet1_name);
        for b = 1:size(bold_cells1,1)
            Location = [xlcolumn(bold_cells1(b,2)) num2str(bold_cells1(b,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Bold = true;
        end
        for s = 1:size(sub_cells1,1)
            Location = [xlcolumn(sub_cells1(s,2)) num2str(sub_cells1(s,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Underline = true;
        end
        % Sheet 2
        sheet = get(Excel.Worksheets,'Item', 2);
        invoke(sheet, 'Activate');
        set(sheet, 'Name', sheet2_name);
        for b = 1:size(bold_cells2,1)
            Location = [xlcolumn(bold_cells2(b,2)) num2str(bold_cells2(b,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Bold = true;
        end
        for p = 1:size(percent_cells2,1)
            Location = [xlcolumn(percent_cells2(p,2)) num2str(percent_cells2(p,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.NumberFormat = '0%';
        end
        % Sheet 3
        sheet = get(Excel.Worksheets,'Item', 3);
        invoke(sheet, 'Activate');
        set(sheet, 'Name', sheet3_name);
        for b = 1:size(bold_cells3,1)
            Location = [xlcolumn(bold_cells3(b,2)) num2str(bold_cells3(b,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Bold = true;
        end
        for p = 1:size(percent_cells3,1)
            Location = [xlcolumn(percent_cells3(p,2)) num2str(percent_cells3(p,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.NumberFormat = '0%';
        end
        
        invoke(Workbook,'Save');
        invoke(Excel,'Quit');
        delete(Excel);
    end
end

end

