%% This function saves detailed results in file "filename" when piles from the same individual must be covered by different summay piles
function [error] = save_detailedOPPSP(filename,raw_data,yilij,results,K,run)
error = '';
try
    if exist(filename,'file')
        disp(['deleting ' filename]);
        delete(filename);
        disp('done.');
    end
catch
    error = ['The file ' filename ' is not writable.'];
    return;
end
%% SHEET 1
bold_cells1 = [];
sub_cells1 = [];
sheet1_name = 'Summary';
sheet1{1,1} = 'Category Covering 1.0 - Detailed Solution Output'; bold_cells1 = [bold_cells1;1,1];
sheet1{3,1} = 'Software by:';
sheet1{4,1} = '(withheld through the review process)';
sheet1{6,1} = ['Solution for ' num2str(K) ' summary piles, execution #' num2str(run) '.']; bold_cells1 = [bold_cells1;6,1];
sheet1{8,1} = 'Key data:'; bold_cells1 = [bold_cells1;8,1];
I = length(raw_data.yilij);
sheet1{9,1} = I; sheet1{9,2} = 'individuals';
sum_ci=0;
for i = 1:I
    sum_ci = sum_ci + size(raw_data.yilij{i},1);
end
sheet1{10,1} = sum_ci; sheet1{10,2} = 'piles made in the data';
yilij_cat = [];
for i=1:I
    yilij_cat = [yilij_cat;yilij{i}];
end
sheet1{11,1} = size(unique(yilij_cat,'rows'),1); sheet1{11,2} = 'unique piles';
J = size(raw_data.yilij{1},2);
sheet1{12,1} = J; sheet1{12,2} = 'objects sorted';
sheet1{13,1} = results.seed(K-results.minK+1,run); sheet1{13,2} = 'Seed';
sheet1{14,1} = results.nIterations(K-results.minK+1,run); sheet1{14,2} = 'Iterations';
sheet1{15,1} = results.Z(K-results.minK+1,run); sheet1{15,2} = 'Total miscoverings';
sheet1{17,1} = 'Summary:'; bold_cells1 = [bold_cells1;17,1];
sheet1{18,1} = 'Key matrices'; sub_cells1 = [sub_cells1;18,1];
sheet1{19,1} = 'Data on the chosen summary piles.';
sheet1{21,1} = 'Data and predicted piles'; sub_cells1 = [sub_cells1;21,1];
sheet1{22,1} = 'Raw data, and as a final column the number of the predicted summary pile.';
sheet1{24,1} = 'ID and Summary Piles'; sub_cells1 = [sub_cells1;24,1];
sheet1{25,1} = 'For each participant, 0/1 variables indicate whether the model estimates the participant was using the summary pile.';
sheet1{26,1} = 'Note: This is particularly helpful to merge with a SPSS post-task survey.';
sheet1{28,1} = 'Labels and Summary Piles'; sub_cells1 = [sub_cells1;28,1];
sheet1{29,1} = 'This sheet provides an easily accessible list of all the labels used to describe the piles associated with each summary pile.';
status = xlswrite(filename,sheet1,1);
if status == 0
    error = ['The file ' filename ' is not writable.  It may be locked by another process.'];
    return;
end


%% SHEET 2
[si,avg] = silhouette(raw_data.yilij,results.pmj(K-results.minK+1,run),results.ximli{K-results.minK+1,run});

bold_cells2 = [];
percent_cells2 = [];
sheet2_name = 'Key matrices';
sheet2{1,2} = 'Membership Table'; bold_cells2 = [bold_cells2;1,2];
sheet2{2,1} = 'Summary Pile Number';
sheet2{2,2} = 'Label';
sheet2{2,3} = 'Covering Rate';
sheet2{2,4} = 'Silhouette Average';
for j = 1:J
    sheet2{2,4+j} = raw_data.products{j};
end
for m = 1:K
    sheet2{2+m,1} = m;
    sheet2{2+m,3} = results.covRate{K-results.minK+1,run}(m); percent_cells2 = [percent_cells2;2+m,3];
    sheet2{2+m,4} = avg(m); percent_cells2 = [percent_cells2;2+m,4];
    for j=1:J
        sheet2{2+m,4+j} = results.pmj{K-results.minK+1,run}(m,j);
    end
end
sheet2{4+K,2} = 'Accuracy Table'; bold_cells2 = [bold_cells2;4+K,2];
sheet2{5+K,2} = 'Label';
sheet2{5+K,3} = 'Covering rate';
sheet2{5+K,4} = 'Silhouette Average';
for j = 1:J
    sheet2{5+K,4+j} = raw_data.products{j};
end
for m = 1:K
    sheet2{5+K+m,1} = m;
    sheet2{5+K+m,3} = results.covRate{K-results.minK+1,run}(m); percent_cells2 = [percent_cells2;5+K+m,3];
    sheet2{5+K+m,4} = avg(m); percent_cells2 = [percent_cells2;5+K+m,4];
    for j=1:J
        if results.truePos{K-results.minK+1,run}(m,j) > 0
            sheet2{5+K+m,4+j} = results.truePos{K-results.minK+1,run}(m,j); bold_cells2 = [bold_cells2;5+K+m,4+j];
        else
            sheet2{5+K+m,4+j} = results.trueNeg{K-results.minK+1,run}(m,j);
        end
    end
end
sheet2{7+2*K,2} = 'Bold indicates true positive.'; bold_cells2 = [bold_cells2;7+2*K,2];
sheet2{8+2*K,2} = 'Regular font indiciates true negative rate.';
status = xlswrite(filename,sheet2,2);
if status == 0
    error = ['The file ' filename ' is not writable.  It may be locked by another process.'];
    return;
end

%% SHEET 3
bold_cells3 = [];
sheet3_name = 'Data and predicted piles';
sheet3{1,1} = 'Raw data - Last column contained the number of the predicted summary pile'; bold_cells3 = [bold_cells3;1,1];
for i=1:size(raw_data.raw,1)
    for j=1:size(raw_data.raw,2)
        sheet3{2+i,j} = raw_data.raw{i,j};
    end
end
sheet3{3,size(raw_data.raw,2)+1} = 'Predicted'; bold_cells3 = [bold_cells3;3,size(raw_data.raw,2)+1];
sheet3{3,size(raw_data.raw,2)+2} = 'Silhouette'; bold_cells3 = [bold_cells3;3,size(raw_data.raw,2)+2];
predicted = cell(I,1);
for i=1:I
    for li = 1:size(yilij{i},1)
        predicted{i}(li) = find(results.ximli{K-results.minK+1,run}{i}(:,li));
    end
end
row = 3;
col = size(raw_data.raw,2)+1;
for i = 1:I
    for li = 1:length(predicted{i})
        row = row+1;
        sheet3{row,col} = predicted{i}(li);
        sheet3{row,col+1} = si{i}(li);
    end
end
status = xlswrite(filename,sheet3,3);
if status == 0
    error = ['The file ' filename ' is not writable.  It may be locked by another process.'];
    return;
end

%% SHEET 4
sheet4_name = 'ID and summary piles';
sheet4{1,1} = 'For each participant:';
sheet4{2,1} = '1 indicates that the participant is estimated to have used the summary pile';
sheet4{3,1} = '0 indiciates that the participant is not estimated to have used the summary pile.';
sheet4{5,2} = 'Summary Pile';
sheet4{6,1} = 'Uniqid';
for m = 1:K
    sheet4{6,1+m} = m;
end
for i = 1:I
    sheet4{6+i,1} = raw_data.IDs(i);
    for m = 1:K
        sheet4{6+i,1+m} = 0;
        for li = 1:length(predicted{i})
            if m == predicted{i}(li)
                sheet4{6+i,1+m} = 1;
                break;
            end
        end
    end
end
status = xlswrite(filename,sheet4,4);
if status == 0
    error = ['The file ' filename ' is not writable.  It may be locked by another process.'];
    return;
end

%% SHEET 5
bold_cells5 = [];
sheet5_name = 'Labels and Summary Piles';
sheet5{1,1} = 'For each summary pile, this sheet provides the labels with the piles associated.'; bold_cells5 = [bold_cells5;1,1];
sheet5{3,1} = 'Summary Pile'; bold_cells5 = [bold_cells5;3,1];
for m = 1:K
    sheet5{5,m} = m; bold_cells5 = [bold_cells5;5,m];
    row = 6;
    for i = 1:length(predicted)
        for li = 1:length(predicted{i})
            if predicted{i}(li) == m
                sheet5{row,m} = raw_data.categories{i}{li};
                row = row+1;
            end
        end
    end
end
status = xlswrite(filename,sheet5,5);
if status == 0
    error = ['The file ' filename ' is not writable.  It may be locked by another process.'];
    return;
end

%% BOLD, UNDERLINE and PERCENTAGE
try
    Excel = actxserver('Excel.Application');
catch
    error = 'Cannot edit cells format. Check if Excel Application is installed in your system.';
    Excel = [];
end
if ~isempty(Excel)
    ExcelVersion = str2num(Excel.Version);
    if ExcelVersion < 12
        error = 'Cannot edit cells format. Your Excel Application version may not support *.xlsx files.';
    else
        Workbook = invoke(Excel.Workbooks,'Open',filename);
%         set(Excel, 'Visible', 1);
        % Sheet 1
        sheet = get(Excel.Worksheets,'Item', 1);
        invoke(sheet, 'Activate');
        set(sheet, 'Name', sheet1_name);
        for b = 1:size(bold_cells1,1)
            Location = [xlcolumn(bold_cells1(b,2)) num2str(bold_cells1(b,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Bold = true;
        end
        for s = 1:size(sub_cells1,1)
            Location = [xlcolumn(sub_cells1(s,2)) num2str(sub_cells1(s,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Underline = true;
        end
        % Sheet 2
        sheet = get(Excel.Worksheets,'Item', 2);
        invoke(sheet, 'Activate');
        set(sheet, 'Name', sheet2_name);
        for b = 1:size(bold_cells2,1)
            Location = [xlcolumn(bold_cells2(b,2)) num2str(bold_cells2(b,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Bold = true;
        end
        for p = 1:size(percent_cells2,1)
            Location = [xlcolumn(percent_cells2(p,2)) num2str(percent_cells2(p,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.NumberFormat = '0%';
        end
        % Sheet 3
        sheet = get(Excel.Worksheets,'Item', 3);
        invoke(sheet, 'Activate');
        set(sheet, 'Name', sheet3_name);
        for b = 1:size(bold_cells3,1)
            Location = [xlcolumn(bold_cells3(b,2)) num2str(bold_cells3(b,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Bold = true;
        end
        % Sheet 4
        sheet = get(Excel.Worksheets,'Item', 4);
        invoke(sheet, 'Activate');
        set(sheet, 'Name', sheet4_name);
        % Sheet 5
        sheet = get(Excel.Worksheets,'Item', 5);
        invoke(sheet, 'Activate');
        set(sheet, 'Name', sheet5_name);
        for b = 1:size(bold_cells5,1)
            Location = [xlcolumn(bold_cells5(b,2)) num2str(bold_cells5(b,1))];
            Range = get(Excel.Activesheet, 'Range', Location);
            Range.Select;
            Excel.Selection.Font.Bold = true;
        end
        
        invoke(Workbook,'Save');
        invoke(Excel,'Quit');
        delete(Excel);
    end
end

end

