%% This function computes the silhouette of each pile
function [s,avg] = silhouette(yilij,pmj,ximli)
%% Inputs:
% yilij: piles
% pmj: summary piles
% ximli: cell where each value equals to 1 if pile li is assigned to summary pile m

%% Outputs:
% s: silhouette values for each pile
% avg: average silhouette of piles covered by each summary pile

%%
I = length(yilij); % # of individuals
K = size(pmj{1},1); % # of summary piles
ci = zeros(I,1); % # of piles per individual

avg = zeros(K,1); % average silhouette for each summary pile

s = cell(1,I);

if K==1
    for i = 1:I
        ci(i) = size(yilij{i},1);
        s{i} = zeros(ci(i),1);
    end
    return;
end

n_piles = zeros(K,1);

for i = 1:I
    ci(i) = size(yilij{i},1);
    s{i} = zeros(ci(i),1);
    for li = 1:ci(i)
        for mm = 1:K
            if ximli{i}(mm,li) == 1
                m = mm;
                break;
            end
        end
        miscovs = sum(abs(repmat(yilij{i}(li,:),K,1)-pmj{1}),2);
        a = miscovs(m);
        b = min(miscovs([1:m-1,m+1:end]));
        s{i}(li) = (b-a)/max(a,b);
        n_piles(m) = n_piles(m) + 1;
        avg(m) = avg(m) + s{i}(li);
    end
end
avg = avg./n_piles;

end
